@@ -612,7 +612,7 @@ htmlAttrDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur,
 
 		while (IS_BLANK(*tmp)) tmp++;
 
-		escaped = xmlURIEscapeStr(tmp, BAD_CAST"@/:=?;#%&");
+		escaped = xmlURIEscapeStr(tmp, BAD_CAST"@/:=?;#%&,");
 		if (escaped != NULL) {
 		    xmlBufferWriteQuotedString(buf->buffer, escaped);
 		    xmlFree(escaped);
